package net.ipksindia

import enums.TransactionType
import model.Teller
import net.ipksindia.model.NeftTransaction
import net.ipksindia.model.TransactionRequest
import net.ipksindia.model.TransferTransaction
import java.time.format.DateTimeFormatter

class TransactionFactory(private val transactionRequest: TransactionRequest, private val teller: Teller) {
    private val bankDccbToSftpMap = mutableMapOf("0015" to "0005", "0003" to "0021")
    private val date = transactionRequest.date.format(DateTimeFormatter.ofPattern("dd-MM-yyyy"))
    private val rrn = transactionRequest.date.format(DateTimeFormatter.ofPattern("ddMM")) + transactionRequest.transactionNumber.takeLast(4)

    private fun createTransferTransaction(): TransferTransaction {
        val bankCode = bankDccbToSftpMap[transactionRequest.dccbCode.padStart(4, '0')] ?: throw ItemNotFoundException(
            "SFTP code",
            transactionRequest.dccbCode
        )
        return TransferTransaction(
            bankCode = bankCode,
            branchCode = transactionRequest.branchCode.padStart(3, '0'),
            cbsTellerId = teller.tellerId,
            cbsTellerUserType = teller.userType,
            queIdType = "5",
            description = "${TransactionType.TRANSFER.code}For Checking",
            priority = "1",
            cbsTellerCapability = teller.capability,
            txnScreenNo = TransactionType.TRANSFER.code.padStart(6, '0'),
            txnAmt = transactionRequest.amount,
            txnDate = date,
            sourceAcctNo = transactionRequest.pacsCurrentAccountNumber,
            cbsSbAcctNo = "",
            destinationAcctNo = transactionRequest.linkedCBSAccountNumber,
            narration = "TRF for NEFT/RTGS",
            sourceTxnNo = TransactionType.TRANSFER.code,
            sourceStat = "A/P",
            apiType = "OUTWARD_QUEUE_POSTING",
            remitterName = transactionRequest.remitterName,
            ifscCode = "ABCD0000000",
            rrn = rrn + "1",
            mobileOrEmail = transactionRequest.mobileNumber,
            remitterAddress = "",
            beneficiaryName = "",
            beneficiaryAddress = "",
            senderAcctType = "",
            beneficiaryAcctType = "",
            beneficiaryBankName = "",
            beneficiaryBranchName = "",
            commissionAmt = "0"
        )

    }

    private fun createNEFTTransaction(): NeftTransaction {
        val bankCode = bankDccbToSftpMap[transactionRequest.dccbCode.padStart(4, '0')] ?: throw ItemNotFoundException("SFTP code for", transactionRequest.dccbCode)
        return NeftTransaction(
            bankCode = bankCode,
            branchCode = transactionRequest.branchCode.padStart(3,'0'),
            cbsTellerId = teller.tellerId,
            cbsTellerUserType = teller.userType,
            queIdType = "5",
            description = "${TransactionType.NEFT.code}For Checking",
            priority = "1",
            cbsTellerCapability = teller.capability,
            txnScreenNo = TransactionType.NEFT.code.padStart(6, '0'),
            txnAmt = transactionRequest.amount,
            txnDate = date,
            sourceAcctNo = transactionRequest.linkedCBSAccountNumber,
            cbsSbAcctNo = "",
            destinationAcctNo = transactionRequest.neftBeneficiaryAccountNumber,
            narration = "TRF for NEFT/RTGS",
            sourceTxnNo = TransactionType.NEFT.code,
            sourceStat = "A/P",
            apiType = "OUTWARD_QUEUE_POSTING",
            remitterName = transactionRequest.remitterName,
            ifscCode = transactionRequest.ifscCode,
            rrn = rrn + "2",
            mobileOrEmail = transactionRequest.mobileNumber,
            remitterAddress = transactionRequest.remitterAddress.take(25),
            beneficiaryName = transactionRequest.beneficiaryName,
            beneficiaryAddress = transactionRequest.beneficiaryAddress.take(25),
            senderAcctType = transactionRequest.senderAcctType,
            beneficiaryAcctType = transactionRequest.beneficiaryAcctType,
            beneficiaryBankName = transactionRequest.beneficiaryBankName,
            beneficiaryBranchName = transactionRequest.beneficiaryBranchName,
            commissionAmt = "0"
        )
    }

    fun createTransactionPair(): Pair<TransferTransaction, NeftTransaction> {
        return Pair(createTransferTransaction(), createNEFTTransaction())
    }
}